@@ -24,6 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <string.h>
 #include <time.h>
 
@@ -177,7 +178,7 @@ gdk_property_get (GdkWindow   *window,
   if (nbytes > 0)
     {
       *data = g_malloc (nbytes+1);
-      memcpy (data, prop->data + offset, nbytes);
+      memcpy (*data, prop->data + offset, nbytes);
       (*data)[nbytes] = 0;
     }
   else
