@@ -18,7 +18,6 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkentrycompletion.h"
 #include "gtkentryprivate.h"
 #include "gtkcelllayout.h"
@@ -36,6 +35,9 @@
 #include "gtksignal.h"
 #include "gtkmarshalers.h"
 
+#include "gtkprivate.h"
+#include "gtkalias.h"
+
 #include <string.h>
 
 
@@ -56,7 +58,9 @@ enum
   PROP_MINIMUM_KEY_LENGTH,
   PROP_TEXT_COLUMN,
   PROP_INLINE_COMPLETION,
-  PROP_POPUP_COMPLETION
+  PROP_POPUP_COMPLETION,
+  PROP_POPUP_SET_WIDTH,
+  PROP_POPUP_SINGLE_MATCH
 };
 
 #define GTK_ENTRY_COMPLETION_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_ENTRY_COMPLETION, GtkEntryCompletionPrivate))
@@ -272,16 +276,16 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
                                                         P_("Completion Model"),
                                                         P_("The model to find matches in"),
                                                         GTK_TYPE_TREE_MODEL,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_MINIMUM_KEY_LENGTH,
-                                   g_param_spec_int ("minimum_key_length",
+                                   g_param_spec_int ("minimum-key-length",
                                                      P_("Minimum Key Length"),
                                                      P_("Minimum length of the search key in order to look up matches"),
                                                      0,
                                                      G_MAXINT,
                                                      1,
-                                                     G_PARAM_READWRITE));
+                                                     GTK_PARAM_READWRITE));
   /**
    * GtkEntryCompletion:text-column:
    *
@@ -291,29 +295,31 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    */
   g_object_class_install_property (object_class,
                                    PROP_TEXT_COLUMN,
-                                   g_param_spec_int ("text_column",
+                                   g_param_spec_int ("text-column",
                                                      P_("Text column"),
                                                      P_("The column of the model containing the strings."),
                                                      -1,
                                                      G_MAXINT,
                                                      -1,
-                                                     G_PARAM_READWRITE));
+                                                     GTK_PARAM_READWRITE));
 
   /**
    * GtkEntryCompletion:inline-completion:
    * 
    * Determines whether the common prefix of the possible completions 
-   * should be inserted automatically in the entry.
+   * should be inserted automatically in the entry. Note that this
+   * requires text-column to be set, even if you are using a custom
+   * match function.
    *
    * Since: 2.6
    **/
   g_object_class_install_property (object_class,
 				   PROP_INLINE_COMPLETION,
-				   g_param_spec_boolean ("inline_completion",
+				   g_param_spec_boolean ("inline-completion",
  							 P_("Inline completion"),
  							 P_("Whether the common prefix should be inserted automatically"),
  							 FALSE,
- 							 G_PARAM_READWRITE));
+ 							 GTK_PARAM_READWRITE));
   /**
    * GtkEntryCompletion:popup-completion:
    * 
@@ -324,11 +330,46 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    **/
   g_object_class_install_property (object_class,
 				   PROP_POPUP_COMPLETION,
-				   g_param_spec_boolean ("popup_completion",
+				   g_param_spec_boolean ("popup-completion",
  							 P_("Popup completion"),
  							 P_("Whether the completions should be shown in a popup window"),
  							 TRUE,
- 							 G_PARAM_READWRITE));
+ 							 GTK_PARAM_READWRITE));
+
+  /**
+   * GtkEntryCompletion:popup-set-width:
+   * 
+   * Determines whether the completions popup window will be
+   * resized to the width of the entry.
+   *
+   * Since: 2.8
+   */
+  g_object_class_install_property (object_class,
+				   PROP_POPUP_SET_WIDTH,
+				   g_param_spec_boolean ("popup-set-width",
+ 							 P_("Popup set width"),
+ 							 P_("If TRUE, the popup window will have the same size as the entry"),
+ 							 TRUE,
+ 							 GTK_PARAM_READWRITE));
+
+  /**
+   * GtkEntryCompletion:popup-single-match:
+   * 
+   * Determines whether the completions popup window will shown
+   * for a single possible completion. You probably want to set
+   * this to %FALSE if you are using 
+   * <link linkend="GtkEntryCompletion--inline-completion">inline 
+   * completion</link>.
+   *
+   * Since: 2.8
+   */
+  g_object_class_install_property (object_class,
+				   PROP_POPUP_SINGLE_MATCH,
+				   g_param_spec_boolean ("popup-single-match",
+ 							 P_("Popup single match"),
+ 							 P_("If TRUE, the popup window will appear for a single match."),
+ 							 TRUE,
+ 							 GTK_PARAM_READWRITE));
 
   g_type_class_add_private (object_class, sizeof (GtkEntryCompletionPrivate));
 }
@@ -361,6 +402,8 @@ gtk_entry_completion_init (GtkEntryCompletion *completion)
   priv->has_completion = FALSE;
   priv->inline_completion = FALSE;
   priv->popup_completion = TRUE;
+  priv->popup_set_width = TRUE;
+  priv->popup_single_match = TRUE;
 
   /* completions */
   priv->filter_model = NULL;
@@ -490,6 +533,14 @@ gtk_entry_completion_set_property (GObject      *object,
 	priv->popup_completion = g_value_get_boolean (value);
         break;
 
+      case PROP_POPUP_SET_WIDTH:
+	priv->popup_set_width = g_value_get_boolean (value);
+        break;
+
+      case PROP_POPUP_SINGLE_MATCH:
+	priv->popup_single_match = g_value_get_boolean (value);
+        break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -527,6 +578,14 @@ gtk_entry_completion_get_property (GObject    *object,
         g_value_set_boolean (value, gtk_entry_completion_get_popup_completion (completion));
         break;
 
+      case PROP_POPUP_SET_WIDTH:
+        g_value_set_boolean (value, gtk_entry_completion_get_popup_set_width (completion));
+        break;
+
+      case PROP_POPUP_SINGLE_MATCH:
+        g_value_set_boolean (value, gtk_entry_completion_get_popup_single_match (completion));
+        break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1022,7 +1081,7 @@ gtk_entry_completion_set_minimum_key_length (GtkEntryCompletion *completion,
     {
       completion->priv->minimum_key_length = length;
      
-      g_object_notify (G_OBJECT (completion), "minimum_key_length");
+      g_object_notify (G_OBJECT (completion), "minimum-key-length");
     }
 }
 
@@ -1073,6 +1132,9 @@ gtk_entry_completion_complete (GtkEntryCompletion *completion)
   g_free (tmp);
 
   gtk_tree_model_filter_refilter (completion->priv->filter_model);
+
+  if (GTK_WIDGET_VISIBLE (completion->priv->popup_window))
+    _gtk_entry_completion_resize_popup (completion);
 }
 
 static void
@@ -1205,7 +1267,7 @@ gtk_entry_completion_set_text_column (GtkEntryCompletion *completion,
                                  cell,
                                  "text", column);
 
-  g_object_notify (G_OBJECT (completion), "text_column");
+  g_object_notify (G_OBJECT (completion), "text-column");
 }
 
 /**
@@ -1259,6 +1321,7 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
   gint matches, items, height, x_border, y_border;
   GdkScreen *screen;
   gint monitor_num;
+  gint vertical_separator;
   GdkRectangle monitor;
   GtkRequisition popup_req;
   GtkRequisition entry_req;
@@ -1279,6 +1342,12 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
   gtk_tree_view_column_cell_get_size (completion->priv->column, NULL,
                                       NULL, NULL, NULL, &height);
 
+  gtk_widget_style_get (GTK_WIDGET (completion->priv->tree_view),
+                        "vertical-separator", &vertical_separator,
+                        NULL);
+
+  height += vertical_separator;
+
   if (items <= 0)
     gtk_widget_hide (completion->priv->scrolled_window);
   else
@@ -1289,7 +1358,12 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
 						  GTK_WIDGET (completion->priv->entry)->window);
   gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
-  width = MIN (completion->priv->entry->allocation.width, monitor.width) - 2 * x_border;
+  if (completion->priv->popup_set_width)
+    width = MIN (completion->priv->entry->allocation.width, monitor.width) - 2 * x_border;
+  else
+    width = -1;
+
+  gtk_tree_view_columns_autosize (GTK_TREE_VIEW (completion->priv->tree_view));
   gtk_widget_set_size_request (completion->priv->tree_view, width, items * height);
 
   /* default on no match */
@@ -1313,7 +1387,8 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
   else if (x + popup_req.width > monitor.x + monitor.width)
     x = monitor.x + monitor.width - popup_req.width;
   
-  if (y + entry_req.height + popup_req.height <= monitor.y + monitor.height)
+  if (y + entry_req.height + popup_req.height <= monitor.y + monitor.height ||
+      y - monitor.y < (monitor.y + monitor.height) - (y + entry_req.height))
     {
       y += entry_req.height;
       above = FALSE;
@@ -1342,6 +1417,7 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
 {
   GtkTreeViewColumn *column;
   GList *renderers;
+  GtkWidget *toplevel;
 
   if (GTK_WIDGET_MAPPED (completion->priv->popup_window))
     return;
@@ -1349,12 +1425,15 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
   if (!GTK_WIDGET_MAPPED (completion->priv->entry))
     return;
 
+  if (!GTK_WIDGET_HAS_FOCUS (completion->priv->entry))
+    return;
+
   completion->priv->ignore_enter = TRUE;
     
   column = gtk_tree_view_get_column (GTK_TREE_VIEW (completion->priv->action_view), 0);
   renderers = gtk_tree_view_column_get_cell_renderers (column);
   gtk_widget_ensure_style (completion->priv->tree_view);
-  g_object_set (GTK_CELL_RENDERER (renderers->data), "cell_background_gdk",
+  g_object_set (GTK_CELL_RENDERER (renderers->data), "cell-background-gdk",
                 &completion->priv->tree_view->style->bg[GTK_STATE_NORMAL],
                 NULL);
   g_list_free (renderers);
@@ -1363,6 +1442,17 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
 
   _gtk_entry_completion_resize_popup (completion);
 
+  toplevel = gtk_widget_get_toplevel (completion->priv->entry);
+  if (GTK_IS_WINDOW (toplevel))
+    gtk_window_group_add_window (_gtk_window_get_group (GTK_WINDOW (toplevel)), 
+				 GTK_WINDOW (completion->priv->popup_window));
+
+  /* prevent the first row being focused */
+  gtk_widget_grab_focus (completion->priv->tree_view);
+
+  gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW (completion->priv->tree_view)));
+  gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW (completion->priv->action_view)));
+
   gtk_widget_show (completion->priv->popup_window);
     
   gtk_grab_add (completion->priv->popup_window);
@@ -1412,8 +1502,12 @@ gtk_entry_completion_compute_prefix (GtkEntryCompletion *completion)
   GtkTreeIter iter;
   gchar *prefix = NULL;
   gboolean valid;
+  const gchar *key;
 
-  const gchar *key = gtk_entry_get_text (GTK_ENTRY (completion->priv->entry));
+  if (completion->priv->text_column < 0)
+    return NULL;
+
+  key = gtk_entry_get_text (GTK_ENTRY (completion->priv->entry));
 
   valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (completion->priv->filter_model),
 					 &iter);
@@ -1462,7 +1556,6 @@ gtk_entry_completion_real_insert_prefix (GtkEntryCompletion *completion,
     {
       gint key_len;
       gint prefix_len;
-      gint pos;
       const gchar *key;
 
       prefix_len = g_utf8_strlen (prefix, -1);
@@ -1472,8 +1565,10 @@ gtk_entry_completion_real_insert_prefix (GtkEntryCompletion *completion,
 
       if (prefix_len > key_len)
 	{
+	  gint pos = prefix_len;
+
 	  gtk_editable_insert_text (GTK_EDITABLE (completion->priv->entry),
-				    prefix + key_len, -1, &pos);
+				    prefix + strlen (key), -1, &pos);
 	  gtk_editable_select_region (GTK_EDITABLE (completion->priv->entry),
 				      key_len, prefix_len);
 
@@ -1533,7 +1628,7 @@ gtk_entry_completion_set_inline_completion (GtkEntryCompletion *completion,
     {
       completion->priv->inline_completion = inline_completion;
 
-      g_object_notify (G_OBJECT (completion), "inline_completion");
+      g_object_notify (G_OBJECT (completion), "inline-completion");
     }
 }
 
@@ -1578,7 +1673,7 @@ gtk_entry_completion_set_popup_completion (GtkEntryCompletion *completion,
     {
       completion->priv->popup_completion = popup_completion;
 
-      g_object_notify (G_OBJECT (completion), "popup_completion");
+      g_object_notify (G_OBJECT (completion), "popup-completion");
     }
 }
 
@@ -1600,3 +1695,103 @@ gtk_entry_completion_get_popup_completion (GtkEntryCompletion *completion)
   
   return completion->priv->popup_completion;
 }
+
+/**
+ * gtk_entry_completion_set_popup_set_width:
+ * @completion: a #GtkEntryCompletion
+ * @popup_set_width: %TRUE to make the width of the popup the same as the entry
+ *
+ * Sets whether the completion popup window will be resized to be the same
+ * width as the entry.
+ *
+ * Since: 2.8
+ */
+void 
+gtk_entry_completion_set_popup_set_width (GtkEntryCompletion *completion,
+					  gboolean            popup_set_width)
+{
+  g_return_if_fail (GTK_IS_ENTRY_COMPLETION (completion));
+  
+  popup_set_width = popup_set_width != FALSE;
+
+  if (completion->priv->popup_set_width != popup_set_width)
+    {
+      completion->priv->popup_set_width = popup_set_width;
+
+      g_object_notify (G_OBJECT (completion), "popup-set-width");
+    }
+}
+
+/**
+ * gtk_entry_completion_get_popup_set_width:
+ * @completion: a #GtkEntryCompletion
+ * 
+ * Returns whether the  completion popup window will be resized to the 
+ * width of the entry.
+ * 
+ * Return value: %TRUE if the popup window will be resized to the width of 
+ *   the entry
+ * 
+ * Since: 2.8
+ **/
+gboolean
+gtk_entry_completion_get_popup_set_width (GtkEntryCompletion *completion)
+{
+  g_return_val_if_fail (GTK_IS_ENTRY_COMPLETION (completion), TRUE);
+  
+  return completion->priv->popup_set_width;
+}
+
+
+/**
+ * gtk_entry_completion_set_popup_single_match:
+ * @completion: a #GtkEntryCompletion
+ * @popup_single_match: %TRUE if the popup should appear even for a single
+ *   match
+ *
+ * Sets whether the completion popup window will appear even if there is
+ * only a single match. You may want to set this to %FALSE if you
+ * are using <link linkend="GtkEntryCompletion--inline-completion">inline
+ * completion</link>.
+ *
+ * Since: 2.8
+ */
+void 
+gtk_entry_completion_set_popup_single_match (GtkEntryCompletion *completion,
+					     gboolean            popup_single_match)
+{
+  g_return_if_fail (GTK_IS_ENTRY_COMPLETION (completion));
+  
+  popup_single_match = popup_single_match != FALSE;
+
+  if (completion->priv->popup_single_match != popup_single_match)
+    {
+      completion->priv->popup_single_match = popup_single_match;
+
+      g_object_notify (G_OBJECT (completion), "popup-single-match");
+    }
+}
+
+/**
+ * gtk_entry_completion_get_popup_single_match:
+ * @completion: a #GtkEntryCompletion
+ * 
+ * Returns whether the completion popup window will appear even if there is
+ * only a single match. 
+ * 
+ * Return value: %TRUE if the popup window will appear regardless of the
+ *    number of matches.
+ * 
+ * Since: 2.8
+ **/
+gboolean
+gtk_entry_completion_get_popup_single_match (GtkEntryCompletion *completion)
+{
+  g_return_val_if_fail (GTK_IS_ENTRY_COMPLETION (completion), TRUE);
+  
+  return completion->priv->popup_single_match;
+}
+
+
+#define __GTK_ENTRY_COMPLETION_C__
+#include "gtkaliasdef.c"
