@@ -1195,13 +1195,14 @@ paint_triangle (GtkHSV      *hsv,
 
 	x_start = MAX (xl - PAD, x);
 	x_end = MIN (xr + PAD, x + width);
+	x_start = MIN (x_start, x_end);
 
 	c = (rl << 16) | (gl << 8) | bl;
 
 	for (xx = x; xx < x_start; xx++)
 	  *p++ = c;
 	  
-	for (xx = x_start; xx < x_end; xx++)
+	for (; xx < x_end; xx++)
 	  {
 	    x_interp = CLAMP (xx, xl, xr);
 		
@@ -1212,7 +1213,7 @@ paint_triangle (GtkHSV      *hsv,
 
 	c = (rr << 16) | (gr << 8) | br;
 
-	for (xx = x_end; xx < x + width; xx++)
+	for (; xx < x + width; xx++)
 	  *p++ = c;
       }
     }
