@@ -7,299 +7,221 @@
 #include <glib-object.h>
 
 G_BEGIN_DECLS
-
 /* enumerations from "gtkaccelgroup.h" */
-
-GType gtk_accel_flags_get_type (void);
+GType gtk_accel_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ACCEL_FLAGS (gtk_accel_flags_get_type())
-
 /* enumerations from "gtkcalendar.h" */
-
-GType gtk_calendar_display_options_get_type (void);
+GType gtk_calendar_display_options_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_CALENDAR_DISPLAY_OPTIONS (gtk_calendar_display_options_get_type())
-
-/* enumerations from "gtkclist.h" */
-
-GType gtk_cell_type_get_type (void);
-#define GTK_TYPE_CELL_TYPE (gtk_cell_type_get_type())
-
-GType gtk_clist_drag_pos_get_type (void);
-#define GTK_TYPE_CLIST_DRAG_POS (gtk_clist_drag_pos_get_type())
-
-GType gtk_button_action_get_type (void);
-#define GTK_TYPE_BUTTON_ACTION (gtk_button_action_get_type())
-
-/* enumerations from "gtkctree.h" */
-
-GType gtk_ctree_pos_get_type (void);
-#define GTK_TYPE_CTREE_POS (gtk_ctree_pos_get_type())
-
-GType gtk_ctree_line_style_get_type (void);
-#define GTK_TYPE_CTREE_LINE_STYLE (gtk_ctree_line_style_get_type())
-
-GType gtk_ctree_expander_style_get_type (void);
-#define GTK_TYPE_CTREE_EXPANDER_STYLE (gtk_ctree_expander_style_get_type())
-
-GType gtk_ctree_expansion_type_get_type (void);
-#define GTK_TYPE_CTREE_EXPANSION_TYPE (gtk_ctree_expansion_type_get_type())
-
 /* enumerations from "gtkcellrenderer.h" */
-
-GType gtk_cell_renderer_state_get_type (void);
+GType gtk_cell_renderer_state_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_CELL_RENDERER_STATE (gtk_cell_renderer_state_get_type())
-
-GType gtk_cell_renderer_mode_get_type (void);
+GType gtk_cell_renderer_mode_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_CELL_RENDERER_MODE (gtk_cell_renderer_mode_get_type())
-
+/* enumerations from "gtkclist.h" */
+GType gtk_cell_type_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_CELL_TYPE (gtk_cell_type_get_type())
+GType gtk_clist_drag_pos_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_CLIST_DRAG_POS (gtk_clist_drag_pos_get_type())
+GType gtk_button_action_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_BUTTON_ACTION (gtk_button_action_get_type())
+/* enumerations from "gtkctree.h" */
+GType gtk_ctree_pos_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_CTREE_POS (gtk_ctree_pos_get_type())
+GType gtk_ctree_line_style_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_CTREE_LINE_STYLE (gtk_ctree_line_style_get_type())
+GType gtk_ctree_expander_style_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_CTREE_EXPANDER_STYLE (gtk_ctree_expander_style_get_type())
+GType gtk_ctree_expansion_type_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_CTREE_EXPANSION_TYPE (gtk_ctree_expansion_type_get_type())
 /* enumerations from "gtkdebug.h" */
-
-GType gtk_debug_flag_get_type (void);
+GType gtk_debug_flag_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_DEBUG_FLAG (gtk_debug_flag_get_type())
-
 /* enumerations from "gtkdialog.h" */
-
-GType gtk_dialog_flags_get_type (void);
+GType gtk_dialog_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_DIALOG_FLAGS (gtk_dialog_flags_get_type())
-
-GType gtk_response_type_get_type (void);
+GType gtk_response_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_RESPONSE_TYPE (gtk_response_type_get_type())
-
 /* enumerations from "gtkdnd.h" */
-
-GType gtk_dest_defaults_get_type (void);
+GType gtk_dest_defaults_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_DEST_DEFAULTS (gtk_dest_defaults_get_type())
-
-GType gtk_target_flags_get_type (void);
+GType gtk_target_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TARGET_FLAGS (gtk_target_flags_get_type())
-
 /* enumerations from "gtkenums.h" */
-
-GType gtk_anchor_type_get_type (void);
+GType gtk_anchor_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ANCHOR_TYPE (gtk_anchor_type_get_type())
-
-GType gtk_arrow_type_get_type (void);
+GType gtk_arrow_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ARROW_TYPE (gtk_arrow_type_get_type())
-
-GType gtk_attach_options_get_type (void);
+GType gtk_attach_options_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ATTACH_OPTIONS (gtk_attach_options_get_type())
-
-GType gtk_button_box_style_get_type (void);
+GType gtk_button_box_style_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_BUTTON_BOX_STYLE (gtk_button_box_style_get_type())
-
-GType gtk_curve_type_get_type (void);
+GType gtk_curve_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_CURVE_TYPE (gtk_curve_type_get_type())
-
-GType gtk_delete_type_get_type (void);
+GType gtk_delete_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_DELETE_TYPE (gtk_delete_type_get_type())
-
-GType gtk_direction_type_get_type (void);
+GType gtk_direction_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_DIRECTION_TYPE (gtk_direction_type_get_type())
-
-GType gtk_expander_style_get_type (void);
+GType gtk_expander_style_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_EXPANDER_STYLE (gtk_expander_style_get_type())
-
-GType gtk_icon_size_get_type (void);
+GType gtk_icon_size_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ICON_SIZE (gtk_icon_size_get_type())
-
-GType gtk_side_type_get_type (void);
+GType gtk_side_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SIDE_TYPE (gtk_side_type_get_type())
-
-GType gtk_text_direction_get_type (void);
+GType gtk_text_direction_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TEXT_DIRECTION (gtk_text_direction_get_type())
-
-GType gtk_justification_get_type (void);
+GType gtk_justification_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_JUSTIFICATION (gtk_justification_get_type())
-
-GType gtk_match_type_get_type (void);
+GType gtk_match_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_MATCH_TYPE (gtk_match_type_get_type())
-
-GType gtk_menu_direction_type_get_type (void);
+GType gtk_menu_direction_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_MENU_DIRECTION_TYPE (gtk_menu_direction_type_get_type())
-
-GType gtk_metric_type_get_type (void);
+GType gtk_metric_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_METRIC_TYPE (gtk_metric_type_get_type())
-
-GType gtk_movement_step_get_type (void);
+GType gtk_movement_step_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_MOVEMENT_STEP (gtk_movement_step_get_type())
-
-GType gtk_orientation_get_type (void);
+GType gtk_scroll_step_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_SCROLL_STEP (gtk_scroll_step_get_type())
+GType gtk_orientation_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ORIENTATION (gtk_orientation_get_type())
-
-GType gtk_corner_type_get_type (void);
+GType gtk_corner_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_CORNER_TYPE (gtk_corner_type_get_type())
-
-GType gtk_pack_type_get_type (void);
+GType gtk_pack_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PACK_TYPE (gtk_pack_type_get_type())
-
-GType gtk_path_priority_type_get_type (void);
+GType gtk_path_priority_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PATH_PRIORITY_TYPE (gtk_path_priority_type_get_type())
-
-GType gtk_path_type_get_type (void);
+GType gtk_path_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PATH_TYPE (gtk_path_type_get_type())
-
-GType gtk_policy_type_get_type (void);
+GType gtk_policy_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_POLICY_TYPE (gtk_policy_type_get_type())
-
-GType gtk_position_type_get_type (void);
+GType gtk_position_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_POSITION_TYPE (gtk_position_type_get_type())
-
-GType gtk_preview_type_get_type (void);
+GType gtk_preview_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PREVIEW_TYPE (gtk_preview_type_get_type())
-
-GType gtk_relief_style_get_type (void);
+GType gtk_relief_style_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_RELIEF_STYLE (gtk_relief_style_get_type())
-
-GType gtk_resize_mode_get_type (void);
+GType gtk_resize_mode_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_RESIZE_MODE (gtk_resize_mode_get_type())
-
-GType gtk_signal_run_type_get_type (void);
+GType gtk_signal_run_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SIGNAL_RUN_TYPE (gtk_signal_run_type_get_type())
-
-GType gtk_scroll_type_get_type (void);
+GType gtk_scroll_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SCROLL_TYPE (gtk_scroll_type_get_type())
-
-GType gtk_selection_mode_get_type (void);
+GType gtk_selection_mode_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SELECTION_MODE (gtk_selection_mode_get_type())
-
-GType gtk_shadow_type_get_type (void);
+GType gtk_shadow_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SHADOW_TYPE (gtk_shadow_type_get_type())
-
-GType gtk_state_type_get_type (void);
+GType gtk_state_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_STATE_TYPE (gtk_state_type_get_type())
-
-GType gtk_submenu_direction_get_type (void);
+GType gtk_submenu_direction_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SUBMENU_DIRECTION (gtk_submenu_direction_get_type())
-
-GType gtk_submenu_placement_get_type (void);
+GType gtk_submenu_placement_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SUBMENU_PLACEMENT (gtk_submenu_placement_get_type())
-
-GType gtk_toolbar_style_get_type (void);
+GType gtk_toolbar_style_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TOOLBAR_STYLE (gtk_toolbar_style_get_type())
-
-GType gtk_update_type_get_type (void);
+GType gtk_update_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_UPDATE_TYPE (gtk_update_type_get_type())
-
-GType gtk_visibility_get_type (void);
+GType gtk_visibility_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_VISIBILITY (gtk_visibility_get_type())
-
-GType gtk_window_position_get_type (void);
+GType gtk_window_position_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_WINDOW_POSITION (gtk_window_position_get_type())
-
-GType gtk_window_type_get_type (void);
+GType gtk_window_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_WINDOW_TYPE (gtk_window_type_get_type())
-
-GType gtk_wrap_mode_get_type (void);
+GType gtk_wrap_mode_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_WRAP_MODE (gtk_wrap_mode_get_type())
-
-GType gtk_sort_type_get_type (void);
+GType gtk_sort_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SORT_TYPE (gtk_sort_type_get_type())
-
+GType gtk_im_preedit_style_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_IM_PREEDIT_STYLE (gtk_im_preedit_style_get_type())
+GType gtk_im_status_style_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_IM_STATUS_STYLE (gtk_im_status_style_get_type())
+GType gtk_pack_direction_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_PACK_DIRECTION (gtk_pack_direction_get_type())
+/* enumerations from "gtkfilechooser.h" */
+GType gtk_file_chooser_action_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_FILE_CHOOSER_ACTION (gtk_file_chooser_action_get_type())
+GType gtk_file_chooser_confirmation_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_FILE_CHOOSER_CONFIRMATION (gtk_file_chooser_confirmation_get_type())
+GType gtk_file_chooser_error_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_FILE_CHOOSER_ERROR (gtk_file_chooser_error_get_type())
+/* enumerations from "gtkfilefilter.h" */
+GType gtk_file_filter_flags_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_FILE_FILTER_FLAGS (gtk_file_filter_flags_get_type())
+/* enumerations from "gtkicontheme.h" */
+GType gtk_icon_lookup_flags_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_ICON_LOOKUP_FLAGS (gtk_icon_lookup_flags_get_type())
+GType gtk_icon_theme_error_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_ICON_THEME_ERROR (gtk_icon_theme_error_get_type())
+/* enumerations from "gtkiconview.h" */
+GType gtk_icon_view_drop_position_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_ICON_VIEW_DROP_POSITION (gtk_icon_view_drop_position_get_type())
 /* enumerations from "gtkimage.h" */
-
-GType gtk_image_type_get_type (void);
+GType gtk_image_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_IMAGE_TYPE (gtk_image_type_get_type())
-
 /* enumerations from "gtkmessagedialog.h" */
-
-GType gtk_message_type_get_type (void);
+GType gtk_message_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_MESSAGE_TYPE (gtk_message_type_get_type())
-
-GType gtk_buttons_type_get_type (void);
+GType gtk_buttons_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_BUTTONS_TYPE (gtk_buttons_type_get_type())
-
 /* enumerations from "gtknotebook.h" */
-
-GType gtk_notebook_tab_get_type (void);
+GType gtk_notebook_tab_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_NOTEBOOK_TAB (gtk_notebook_tab_get_type())
-
 /* enumerations from "gtkobject.h" */
-
-GType gtk_object_flags_get_type (void);
+GType gtk_object_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_OBJECT_FLAGS (gtk_object_flags_get_type())
-
-GType gtk_arg_flags_get_type (void);
+GType gtk_arg_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ARG_FLAGS (gtk_arg_flags_get_type())
-
 /* enumerations from "gtkprivate.h" */
-
-GType gtk_private_flags_get_type (void);
+GType gtk_private_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PRIVATE_FLAGS (gtk_private_flags_get_type())
-
 /* enumerations from "gtkprogressbar.h" */
-
-GType gtk_progress_bar_style_get_type (void);
+GType gtk_progress_bar_style_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PROGRESS_BAR_STYLE (gtk_progress_bar_style_get_type())
-
-GType gtk_progress_bar_orientation_get_type (void);
+GType gtk_progress_bar_orientation_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PROGRESS_BAR_ORIENTATION (gtk_progress_bar_orientation_get_type())
-
 /* enumerations from "gtkrc.h" */
-
-GType gtk_rc_flags_get_type (void);
+GType gtk_rc_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_RC_FLAGS (gtk_rc_flags_get_type())
-
-GType gtk_rc_token_type_get_type (void);
+GType gtk_rc_token_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_RC_TOKEN_TYPE (gtk_rc_token_type_get_type())
-
 /* enumerations from "gtksizegroup.h" */
-
-GType gtk_size_group_mode_get_type (void);
+GType gtk_size_group_mode_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SIZE_GROUP_MODE (gtk_size_group_mode_get_type())
-
 /* enumerations from "gtkspinbutton.h" */
-
-GType gtk_spin_button_update_policy_get_type (void);
+GType gtk_spin_button_update_policy_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SPIN_BUTTON_UPDATE_POLICY (gtk_spin_button_update_policy_get_type())
-
-GType gtk_spin_type_get_type (void);
+GType gtk_spin_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SPIN_TYPE (gtk_spin_type_get_type())
-
 /* enumerations from "gtktextiter.h" */
-
-GType gtk_text_search_flags_get_type (void);
+GType gtk_text_search_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TEXT_SEARCH_FLAGS (gtk_text_search_flags_get_type())
-
 /* enumerations from "gtktextview.h" */
-
-GType gtk_text_window_type_get_type (void);
+GType gtk_text_window_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TEXT_WINDOW_TYPE (gtk_text_window_type_get_type())
-
 /* enumerations from "gtktoolbar.h" */
-
-GType gtk_toolbar_child_type_get_type (void);
+GType gtk_toolbar_child_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TOOLBAR_CHILD_TYPE (gtk_toolbar_child_type_get_type())
-
-GType gtk_toolbar_space_style_get_type (void);
+GType gtk_toolbar_space_style_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TOOLBAR_SPACE_STYLE (gtk_toolbar_space_style_get_type())
-
 /* enumerations from "gtktree.h" */
-
-GType gtk_tree_view_mode_get_type (void);
+GType gtk_tree_view_mode_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TREE_VIEW_MODE (gtk_tree_view_mode_get_type())
-
 /* enumerations from "gtktreemodel.h" */
-
-GType gtk_tree_model_flags_get_type (void);
+GType gtk_tree_model_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TREE_MODEL_FLAGS (gtk_tree_model_flags_get_type())
-
 /* enumerations from "gtktreeview.h" */
-
-GType gtk_tree_view_drop_position_get_type (void);
+GType gtk_tree_view_drop_position_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TREE_VIEW_DROP_POSITION (gtk_tree_view_drop_position_get_type())
-
 /* enumerations from "gtktreeviewcolumn.h" */
-
-GType gtk_tree_view_column_sizing_get_type (void);
+GType gtk_tree_view_column_sizing_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TREE_VIEW_COLUMN_SIZING (gtk_tree_view_column_sizing_get_type())
-
+/* enumerations from "gtkuimanager.h" */
+GType gtk_ui_manager_item_type_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_UI_MANAGER_ITEM_TYPE (gtk_ui_manager_item_type_get_type())
 /* enumerations from "gtkwidget.h" */
-
-GType gtk_widget_flags_get_type (void);
+GType gtk_widget_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_WIDGET_FLAGS (gtk_widget_flags_get_type())
-
-GType gtk_widget_help_type_get_type (void);
+GType gtk_widget_help_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_WIDGET_HELP_TYPE (gtk_widget_help_type_get_type())
-
 G_END_DECLS
 
 #endif /* __GTK_TYPE_BUILTINS_H__ */
